% File: test_mhmm_full_image.m

% Author: Shawn Hymel
% Date: December 10, 2011
% ECE 5554 - Computer Vision

% This script tests the Forward-Backward Algorithm on a sample full image
% given a text file containing continuous value HMM parameters. I till
% highlight the areas that exceed the likelihood threshold of matching the
% given HMM parameters.

close all
clear all

%% Parameters
hmm_file = 'models/hmm_waldo.txt';
image_file = 'images/test/test-01.JPG';
sub_window = [40 20];   % Size of window to search
obs_window = [5 20];    % Size of sampling window to construct obs vector
num_dct_coeffs = 3;     % No. of DCT coefficients to extract for each color
p_thresh = -1000;       % Threshold for match

%% Read HMM parameters
[A,Pi,Mu,Sigma,W,N,D,G] = read_mhmm(hmm_file);

%% Scan image for matches

% Read image
im_info = imfinfo(image_file);
im = imread(image_file);
im = double(im)/256;

% Display test image
figure
imshow(im);

% Extract image size info
sub_width = sub_window(2);
sub_height = sub_window(1);
obs_width = obs_window(2);
obs_height = obs_window(1);
im_width = im_info.Width;
im_height = im_info.Height;

% Scan through image comparing each sub image to the HMM
obs_length = sub_height - obs_height + 1;
obs = zeros(num_dct_coeffs * 3, obs_length);
h_wait_bar = waitbar(0, 'Finding Waldo...');
hits = zeros(1,3);
for y = 1:1:(im_height - sub_height)
    for x = 1:1:(im_width - sub_width)
        
        % Extract sub image
        sub_image = im(y:1:(y+sub_height), x:1:(x+sub_width), :);
        
        for t = 1:1:obs_length
            
            % Find DCT of red, green, blue separately
            im_dct_r = dct2(sub_image(t:t+obs_height-1,1:obs_width,1));
            im_dct_g = dct2(sub_image(t:t+obs_height-1,1:obs_width,2));
            im_dct_b = dct2(sub_image(t:t+obs_height-1,1:obs_width,3));
            
            % Extract desired coefficients from DCT
            dct_c_r = extract_dct_coeff(im_dct_r, num_dct_coeffs);
            dct_c_g = extract_dct_coeff(im_dct_g, num_dct_coeffs);
            dct_c_b = extract_dct_coeff(im_dct_b, num_dct_coeffs); 
            
            % Combine DCT coefficients into one observation vector
            dct_c = [dct_c_r dct_c_g dct_c_b];
            obs(:,t) = dct_c';
        
        end
        
        % Calculate P[O|HMM] of the sub image using Fwd-Bck Algorithm
        [loglik, errors] = mhmm_logprob(obs, Pi, A, Mu, Sigma, W);
        
        % Compare to threshold and store coordinates
        if loglik > p_thresh
            hits = [hits; x y loglik];
        end

    end
    waitbar(y/(im_height - sub_height), h_wait_bar);
end

% Remove dummy entry from hits array
hits = hits(2:end,:);

% Close wait bar
close(h_wait_bar);

% Highlight the hits
highlight_hits(im, hits, sub_window)