% File: test_mhmm_sub_image.m

% Author: Shawn Hymel
% Date: December 9, 2011
% ECE 5554 - Computer Vision

% This script tests the Forward-Backward Algorithm on a sample sub-image
% given a text file containing continuous value HMM parameters.

close all
clear all

%% Parameters
hmm_file = 'models/hmm_waldo.txt';
image_file = 'images/sub/waldo-cal-02-lq.jpg';
sub_window = [5 20];    % Size of sampling window to construct obs vector
num_dct_coeffs = 3;     % Number of DCT coefficients to extract

%% Read HMM parameters
[A,Pi,Mu,Sigma,W,N,D,G] = read_mhmm(hmm_file);

%% Sample Image

% Read test image
im_info = imfinfo(image_file);
im = imread(image_file);
im = double(im)/256;

% Display test image
figure
imshow(im);

% Calculate DCT for each window by scanning down the image
obs_length = im_info.Height - sub_window(1) + 1;
obs = zeros(obs_length, num_dct_coeffs * 3);
for t = 1:1:obs_length
    
    % Find DCT of red, green, blue separately
    im_dct_r = dct2(im(t:t+sub_window(1)-1,1:sub_window(2),1));
    im_dct_g = dct2(im(t:t+sub_window(1)-1,1:sub_window(2),2));
    im_dct_b = dct2(im(t:t+sub_window(1)-1,1:sub_window(2),3));

    % Extract desired coefficients from DCT
    dct_c_r = extract_dct_coeff(im_dct_r, num_dct_coeffs);
    dct_c_g = extract_dct_coeff(im_dct_g, num_dct_coeffs);
    dct_c_b = extract_dct_coeff(im_dct_b, num_dct_coeffs);

    % Combine DCT coefficients into one observation vector
    dct_c = [dct_c_r dct_c_g dct_c_b];
    obs(t,:) = dct_c;
    
end

% Plot observation sequence from test image
figure
surf(obs);

%% Calculate P[O|HMM] Using Fwd-Bck Algorithm
obs = obs';
[loglik, errors] = mhmm_logprob(obs, Pi, A, Mu, Sigma, W);
disp(['Log likelihood: ', num2str(loglik)])